#include "com_geometros_JavaDemo_SceneView.h"
#include <GLES2/gl2.h>
#include <GLES2/gl2ext.h>
#include "shader.h"
#include "sgCore/sgCore.h"
#include "sgCore/sg3D.h"
#include "utils.h"
#include <math.h>
#include <string>
#include "scenes.h"
#include "iterator.h"
#include <android/log.h>

#define  LOG_INFO(...)  __android_log_print(ANDROID_LOG_INFO,"NATIVE",__VA_ARGS__)

std::string JavaStringToStdString(JNIEnv * Env, jstring javaString)
{
   const char *nativeString = Env->GetStringUTFChars(javaString, 0);
   std::string Res = nativeString;
   Env->ReleaseStringUTFChars(javaString, nativeString);
   return Res;
}

bool KernelReady = false;
cCriticalSection* Lock = NULL;
std::string tempPath;

JNIEXPORT void JNICALL Java_com_geometros_javademo_SceneView_nativeInit(JNIEnv *Env, jclass, jstring TempDir){
	if (Lock==NULL)
		Lock = new cCriticalSection();
	Lock->Lock();
	tempPath = JavaStringToStdString(Env,TempDir);
	if (!KernelReady){
		sgInitKernel(tempPath.c_str());

		sgCFont* fnt1 = sgCFont::LoadFont((tempPath+"/Helvetica.shx").c_str(),NULL,0);
		if (fnt1)
			if (!sgFontManager::AttachFont(fnt1))
				sgCFont::UnloadFont(fnt1);

		sgFontManager::SetCurrentFont(0);

		KernelReady = true;
	}
	Lock->Unlock();
}

static unsigned int s_FloorTexture;
static cDemoFloorShader s_FloorShader;
static cDemoSceneShader s_SceneShader;

static float quad[12] = {0,0, 1,0, 0,1, 0,1, 1,0, 1,1};

JNIEXPORT void JNICALL Java_com_geometros_javademo_SceneView_nativeReinitGL(JNIEnv *, jclass, jint shaderScene, jint shaderFloor, jint floorTexture){
	Lock->Lock();
	s_FloorTexture = floorTexture;
	s_FloorShader.setProgram(shaderFloor);
	s_SceneShader.setProgram(shaderScene);
	Lock->Unlock();
}
JNIEXPORT void JNICALL Java_com_geometros_javademo_SceneView_nativeRelease(JNIEnv *, jclass){
	Lock->Lock();
	sgFreeKernel(true);
	KernelReady = false;
	Lock->Unlock();
}
JNIEXPORT void JNICALL Java_com_geometros_javademo_SceneView_nativeClear(JNIEnv *, jclass){
	Lock->Lock();
	Lock->Unlock();
}

SG_POINT toPoint(float x, float y, float z){
	SG_POINT Result;
	Result.x = x;
	Result.y = y;
	Result.z = z;
	return Result;
}

/*
SG_OT_BAD_OBJECT,
	SG_OT_POINT,
	SG_OT_LINE,
	SG_OT_CIRCLE,
	SG_OT_ARC,
	SG_OT_SPLINE,
	SG_OT_TEXT,
	SG_OT_CONTOUR,
	SG_OT_DIM,
	SG_OT_3D
 */

void drawObjects(IObjectsList* objects, const float* identityMatrix){
	sgCObject* obj =  objects->GetHead();

	while (obj!=NULL){

		int type = obj->GetType();
				if (type==SG_OT_POINT){
					sgCPoint* point = (sgCPoint*)obj;
					if (point->GetTempMatrix()!=NULL)
						s_SceneShader.setObjectParams(point->GetTempMatrix()->GetTransparentData(), 0,0,0,1, false);
					else
						s_SceneShader.setObjectParams(identityMatrix, 0,0,0,1, false);

					glVertexAttribPointer(s_SceneShader.getPositionAttrib(), 3, GL_FLOAT, GL_FALSE, sizeof(float)*3, point->GetGeometry());
					glEnableVertexAttribArray(s_SceneShader.getPositionAttrib());
					glDrawArrays(GL_POINTS,0,1);
					glDisableVertexAttribArray(s_SceneShader.getPositionAttrib());
				}
				else if (type==SG_OT_LINE){
					sgCLine* line =(sgCLine*)obj;
					if (line->GetTempMatrix()!=NULL)
						s_SceneShader.setObjectParams(line->GetTempMatrix()->GetTransparentData(), 0,0,0,1, false);
					else
						s_SceneShader.setObjectParams(identityMatrix, 0,0,0,1, false);

					glVertexAttribPointer(s_SceneShader.getPositionAttrib(), 3, GL_FLOAT, GL_FALSE, sizeof(float)*3, line->GetGeometry());
					glEnableVertexAttribArray(s_SceneShader.getPositionAttrib());
					glDrawArrays(GL_LINES,0,2);
					glDisableVertexAttribArray(s_SceneShader.getPositionAttrib());
				}
				else if (type==SG_OT_CIRCLE){
					sgCCircle* circle =(sgCCircle*)obj;
					if (circle->GetTempMatrix()!=NULL)
						s_SceneShader.setObjectParams(circle->GetTempMatrix()->GetTransparentData(), 0,0,0,1, false);
					else
						s_SceneShader.setObjectParams(identityMatrix, 0,0,0,1, false);

					glVertexAttribPointer(s_SceneShader.getPositionAttrib(), 3, GL_FLOAT, GL_FALSE, sizeof(float)*3, circle->GetPoints());
					glEnableVertexAttribArray(s_SceneShader.getPositionAttrib());
					glDrawArrays(GL_LINE_STRIP,0,circle->GetPointsCount());
					glDisableVertexAttribArray(s_SceneShader.getPositionAttrib());
				}
				else if (type==SG_OT_ARC){
					sgCArc* arc =(sgCArc*)obj;
					if (arc->GetTempMatrix()!=NULL)
						s_SceneShader.setObjectParams(arc->GetTempMatrix()->GetTransparentData(), 0,0,0,1, false);
					else
						s_SceneShader.setObjectParams(identityMatrix, 0,0,0,1, false);

					glVertexAttribPointer(s_SceneShader.getPositionAttrib(), 3, GL_FLOAT, GL_FALSE, sizeof(float)*3, arc->GetPoints());
					glEnableVertexAttribArray(s_SceneShader.getPositionAttrib());
					glDrawArrays(GL_LINE_STRIP,0,arc->GetPointsCount());
					glDisableVertexAttribArray(s_SceneShader.getPositionAttrib());
				}
				else if (type==SG_OT_SPLINE){
					sgCSpline* spline = (sgCSpline*)obj;
					if (spline->GetTempMatrix()!=NULL)
						s_SceneShader.setObjectParams(spline->GetTempMatrix()->GetTransparentData(), 0,0,0,1, false);
					else
						s_SceneShader.setObjectParams(identityMatrix, 0,0,0,1, false);

					const SG_SPLINE* data = spline->GetGeometry();

					glVertexAttribPointer(s_SceneShader.getPositionAttrib(), 3, GL_FLOAT, GL_FALSE, sizeof(float)*3, data->GetPoints());
					glEnableVertexAttribArray(s_SceneShader.getPositionAttrib());
					glDrawArrays(GL_LINE_STRIP,0,data->GetPointsCount());
					glDisableVertexAttribArray(s_SceneShader.getPositionAttrib());
				}
				else if (type==SG_OT_TEXT){
					sgCText* text =(sgCText*)obj;
					if (text->GetTempMatrix()!=NULL)
						s_SceneShader.setObjectParams(text->GetTempMatrix()->GetTransparentData(), 0,0,0,1, false);
					else
						s_SceneShader.setObjectParams(identityMatrix, 0,0,0,1, false);

					glVertexAttribPointer(s_SceneShader.getPositionAttrib(), 3, GL_FLOAT, GL_FALSE, sizeof(float)*3, text->GetLines());
					glEnableVertexAttribArray(s_SceneShader.getPositionAttrib());
					glDrawArrays(GL_LINES,0,text->GetLinesCount()*2);
					glDisableVertexAttribArray(s_SceneShader.getPositionAttrib());
				}
				else if (type==SG_OT_CONTOUR){
					sgCContour* contour = (sgCContour*)obj;
					drawObjects(contour->GetChildrenList(), identityMatrix);
				}
				else if (type==SG_OT_DIM){
					sgCDimensions* dim =(sgCDimensions*)obj;
					if (dim->GetTempMatrix()!=NULL)
						s_SceneShader.setObjectParams(dim->GetTempMatrix()->GetTransparentData(), 0,0,0,1, false);
					else
						s_SceneShader.setObjectParams(identityMatrix, 0,0,0,1, false);

					glVertexAttribPointer(s_SceneShader.getPositionAttrib(), 3, GL_FLOAT, GL_FALSE, sizeof(float)*3, dim->GetLines());
					glEnableVertexAttribArray(s_SceneShader.getPositionAttrib());
					glDrawArrays(GL_LINES,0,dim->GetLinesCount()*2);
					glDisableVertexAttribArray(s_SceneShader.getPositionAttrib());
				}
				else if (type==SG_OT_3D){
					sgC3DObject* obj3D = (sgC3DObject*)obj;

					const SG_ALL_TRIANGLES* triangles = obj3D->GetTriangles();
					if (triangles==NULL){
						obj3D->Triangulate(SG_VERTEX_TRIANGULATION,SG_SEPARATE_BUFFERS);
					}
					else
					{
						SG_POINT Color = COLORS[obj3D->GetAttribute(SG_OA_COLOR)];

						s_SceneShader.setObjectParams(obj3D->GetWorldMatrixData(), Color.x,Color.y,Color.z,1, true);

						glVertexAttribPointer(s_SceneShader.getPositionAttrib(), 3, GL_FLOAT, GL_FALSE, sizeof(float)*3, triangles->allVertex);
						glEnableVertexAttribArray(s_SceneShader.getPositionAttrib());
						glVertexAttribPointer(s_SceneShader.getNormalAttrib(), 3, GL_FLOAT, GL_FALSE, sizeof(float)*3, triangles->allNormals);
						glEnableVertexAttribArray(s_SceneShader.getNormalAttrib());
						glDrawArrays(GL_TRIANGLES,0,triangles->nTr*3);
						glDisableVertexAttribArray(s_SceneShader.getPositionAttrib());
						glDisableVertexAttribArray(s_SceneShader.getNormalAttrib());
					}
				}
				else if (type==SG_OT_GROUP){
					sgCGroup* group = (sgCGroup*)obj;
					drawObjects(group->GetChildrenList(),identityMatrix);
				}
				else if (type==SG_OT_BAD_OBJECT){
					LOG_INFO("INCORRECT OBJECT");
				}
				else
					LOG_INFO("unknown object type=%d",obj->GetType());

		obj = objects->GetNext(obj);
	}
}

static float tt[18] = {0,0,0, 100,0,0, 0,100,0, 0,100,0, 100,0,0, 100,100,0};

JNIEXPORT void JNICALL Java_com_geometros_javademo_SceneView_nativeDraw(JNIEnv *, jclass, jint width, jint height, jfloat xAngle, jfloat yAngle){
	Lock->Lock();
	glViewport(0,0,width,height);
	glClearColor(1,1,1,1);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);



	sgCMatrix identity;
	identity.Identity();

	sgCMatrixGL Projection;
	Projection.SetPerspective(45.f, (float)width/height,10,1200.0f);

	SG_POINT axisX = toPoint(1,0,0);
	SG_POINT axisZ = toPoint(0,0,1);
	SG_POINT center = toPoint(0,0,5);
	SG_POINT shift = toPoint(0,0,-800);

	sgCMatrixGL View;
	View.Identity();
	View.Rotate(center,axisZ, -xAngle/360.f*3.1415);
	View.Rotate(center,axisX, -yAngle/360.f*3.1415);
	View.Translate(shift);


	sgCMatrixGL ViewProjection(View);

	ViewProjection.Multiply(Projection);
	ViewProjection.Transpose();

	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
	glEnable(GL_BLEND);
	s_FloorShader.bind(ViewProjection.GetData());
	glVertexAttribPointer(s_FloorShader.getTexCoordAttrib(), 2, GL_FLOAT, GL_FALSE, sizeof(float)*2, quad);
	glEnableVertexAttribArray(s_FloorShader.getTexCoordAttrib());
	glDrawArrays(GL_TRIANGLES,0,6);
	glDisableVertexAttribArray(s_FloorShader.getTexCoordAttrib());
	glDisable(GL_BLEND);

	glDisable(GL_DEPTH_TEST);


	s_SceneShader.bind(ViewProjection.GetData());

	glEnable(GL_DEPTH_TEST);
	drawObjects(sgGetScene()->GetObjectsList(), identity.GetData());
	glDisable(GL_DEPTH_TEST);





	Lock->Unlock();
}
JNIEXPORT void JNICALL Java_com_geometros_javademo_SceneView_nativeLoadScene(JNIEnv *, jclass, jint id){
	LOG_INFO("LOAD SCENE %d", id);
	Lock->Lock();
	sgGetScene()->Clear();
	switch(id){
		case 0: loadScene0(); break;
		case 1: loadScene1(); break;
		case 2: loadScene2(); break;
		case 3: loadScene3(); break;
		case 4: loadScene4(); break;
		case 5: loadScene5(); break;
		case 6: loadScene6(); break;
		case 7: loadScene7(); break;
		case 8: loadScene8(); break;
		case 9: loadScene9(); break;
		case 10: loadScene10(); break;
		case 11: loadScene11((tempPath+"/statue.dxf").c_str()); break;
		case 12: loadScene12((tempPath+"/dino.stl").c_str()); break;
	}
	Lock->Unlock();
}
