/*
 * utils.h
 *
 *  Created on: 02 . 2014 .
 *      Author: Allex
 */

#ifndef UTILS_H_
#define UTILS_H_

#include "sgCore/sgCore.h"
#include <math.h>

#include <pthread.h>

class cCriticalSection
{
protected:
	mutable pthread_mutex_t TheMutex;
public:
	cCriticalSection()
    {
        pthread_mutex_init( &TheMutex, 0 );
    }

    void Lock() const
    {
        pthread_mutex_lock( &TheMutex );
    }

    void Unlock() const
    {
        pthread_mutex_unlock( &TheMutex );
    }

    ~cCriticalSection()
    {
        pthread_mutex_destroy( &TheMutex );
    }
};

class sgCMatrixGL: public sgCMatrix{
protected:
	void swapItems(int i1, int i2){
		sgFloat tmp = m_matrix[i1];
		m_matrix[i1] = m_matrix[i2];
		m_matrix[i2] = tmp;
	}
public:
	void SetPerspective(float fovY, float aspect, float zNear, float zFar){
		float sine, cotangent, deltaZ;
		float radians=(float)(fovY/2.0f*3.1415f/180.0f);

		deltaZ=zFar-zNear;
		sine=(float)sin(radians);
		if ((deltaZ==0.0f) || (sine==0.0f) || (aspect==0.0f))
		{
			return;
		}
		cotangent=(float)(cos(radians)/sine);

		Identity();

		m_matrix[0] = cotangent / aspect;
		m_matrix[5] = cotangent;
		m_matrix[10] = -(zFar + zNear) / deltaZ;
		m_matrix[14] = -1.0f;
		m_matrix[11] = -2.0f * zNear * zFar / deltaZ;
		m_matrix[15] = 0;
	}

	void Transpose(){
		swapItems(1,4);
		swapItems(2,8);
		swapItems(3,12);
		swapItems(6,9);
		swapItems(7,13);
		swapItems(11,14);
	}
};

#endif /* UTILS_H_ */
