//
// File:		BasicOpenGLView.m
//
// Abstract:	Basic OpenGL View with Renderer information
//
// Version:		1.1 - minor fixes.
//				1.0 - Original release.
//				
//
// Disclaimer:	IMPORTANT:  This Apple software is supplied to you by Apple Inc. ("Apple")
//				in consideration of your agreement to the following terms, and your use,
//				installation, modification or redistribution of this Apple software
//				constitutes acceptance of these terms.  If you do not agree with these
//				terms, please do not use, install, modify or redistribute this Apple
//				software.
//
//				In consideration of your agreement to abide by the following terms, and
//				subject to these terms, Apple grants you a personal, non - exclusive
//				license, under Apple's copyrights in this original Apple software ( the
//				"Apple Software" ), to use, reproduce, modify and redistribute the Apple
//				Software, with or without modifications, in source and / or binary forms;
//				provided that if you redistribute the Apple Software in its entirety and
//				without modifications, you must retain this notice and the following text
//				and disclaimers in all such redistributions of the Apple Software. Neither
//				the name, trademarks, service marks or logos of Apple Inc. may be used to
//				endorse or promote products derived from the Apple Software without specific
//				prior written permission from Apple.  Except as expressly stated in this
//				notice, no other rights or licenses, express or implied, are granted by
//				Apple herein, including but not limited to any patent rights that may be
//				infringed by your derivative works or by other works in which the Apple
//				Software may be incorporated.
//
//				The Apple Software is provided by Apple on an "AS IS" basis.  APPLE MAKES NO
//				WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED
//				WARRANTIES OF NON - INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A
//				PARTICULAR PURPOSE, REGARDING THE APPLE SOFTWARE OR ITS USE AND OPERATION
//				ALONE OR IN COMBINATION WITH YOUR PRODUCTS.
//
//				IN NO EVENT SHALL APPLE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR
//				CONSEQUENTIAL DAMAGES ( INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
//				SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
//				INTERRUPTION ) ARISING IN ANY WAY OUT OF THE USE, REPRODUCTION, MODIFICATION
//				AND / OR DISTRIBUTION OF THE APPLE SOFTWARE, HOWEVER CAUSED AND WHETHER
//				UNDER THEORY OF CONTRACT, TORT ( INCLUDING NEGLIGENCE ), STRICT LIABILITY OR
//				OTHERWISE, EVEN IF APPLE HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// Copyright ( C ) 2003-2007 Apple Inc. All Rights Reserved.
//


#import <OpenGL/gl.h>
#import <OpenGL/glext.h>
#import <OpenGL/glu.h>

#import "3DCamera.h"

typedef struct {
	int cx;
	int cy;
} SIZE;

@interface BasicOpenGLView : NSOpenGLView
{
		
	IBOutlet NSMenuItem * primitivesPointsMenuItem;
	IBOutlet NSMenuItem * primitivesLinesMenuItem;
	IBOutlet NSMenuItem * primitivesCirclesMenuItem;
	IBOutlet NSMenuItem * primitivesArcsMenuItem;
	IBOutlet NSMenuItem * primitivesSplinesMenuItem;
	IBOutlet NSMenuItem * primitivesContoursMenuItem;
	IBOutlet NSMenuItem * primitivesEquidistansMenuItem;
	
	IBOutlet NSMenuItem * primitivesBoxesMenuItem;
	IBOutlet NSMenuItem * primitivesSpheresMenuItem;
	IBOutlet NSMenuItem * primitivesCylindersMenuItem;
	IBOutlet NSMenuItem * primitivesConesMenuItem;
	IBOutlet NSMenuItem * primitivesEllipsoidsMenuItem;
	IBOutlet NSMenuItem * primitivesTorsMenuItem;
	IBOutlet NSMenuItem * primitivesSphericBandsMenuItem;
	
	IBOutlet NSMenuItem * booleanIntersectionMenuItem;
	IBOutlet NSMenuItem * booleanUnionMenuItem;
	IBOutlet NSMenuItem * booleanSubMenuItem;
	IBOutlet NSMenuItem * booleanIntersectionContoursMenuItem;
	IBOutlet NSMenuItem * booleanSectionsMenuItem;	
	
	IBOutlet NSMenuItem * kinematicRotationSurfaceMenuItem;
	IBOutlet NSMenuItem * kinematicRotationBodyMenuItem;
	IBOutlet NSMenuItem * kinematicExtrudeSurfaceMenuItem;	
	IBOutlet NSMenuItem * kinematicExtrudeBodyMenuItem;
	IBOutlet NSMenuItem * kinematicSpiralSurfaceMenuItem;
	IBOutlet NSMenuItem * kinematicSpiralBodyMenuItem;	
	IBOutlet NSMenuItem * kinematicPipeSurfaceMenuItem;
	IBOutlet NSMenuItem * kinematicPipeBodyMenuItem;
	
	IBOutlet NSMenuItem * surfacesMeshMenuItem;
	IBOutlet NSMenuItem * surfacesFaceMenuItem;
	IBOutlet NSMenuItem * surfacesCoons3MenuItem;		
	IBOutlet NSMenuItem * surfacesCoons4MenuItem;	
	IBOutlet NSMenuItem * surfacesLinearMenuItem;	
	IBOutlet NSMenuItem * surfacesFromClipsMenuItem;	
	IBOutlet NSMenuItem * surfacesBodyFromClipsMenuItem;	
	
	IBOutlet NSMenuItem * scenesRoomMenuItem;	
	IBOutlet NSMenuItem * scenesDrillsMenuItem;
	IBOutlet NSMenuItem * scenesClockMenuItem;	
			
	C3dCamera   m_camera;
	int         m_capture_x;
	int         m_capture_y;
	
}

+ (NSOpenGLPixelFormat*) basicPixelFormat;

- (void) updateProjection;
- (void) updateModelView;
- (void) resizeGL;
- (void) resetCamera;

-(IBAction)  primitivesPoints: (id) sender;
-(IBAction)  primitivesLines: (id) sender;
-(IBAction)  primitivesCircles: (id) sender;
-(IBAction)  primitivesArcs: (id) sender;
-(IBAction)  primitivesSplines: (id) sender;
-(IBAction)  primitivesContours: (id) sender;
-(IBAction)  primitivesEquidistans: (id) sender;
-(IBAction)  primitivesBoxes: (id) sender;
-(IBAction)  primitivesSpheres: (id) sender;
-(IBAction)  primitivesCylinders: (id) sender;
-(IBAction)  primitivesCones: (id) sender;
-(IBAction)  primitivesEllipsoids: (id) sender;
-(IBAction)  primitivesTors: (id) sender;
-(IBAction)  primitivesSphericBands: (id) sender;

-(IBAction)  booleanIntersection: (id) sender;
-(IBAction)  booleanUnion: (id) sender;
-(IBAction)  booleanSub: (id) sender;
-(IBAction)  booleanIntersectionContours: (id) sender;
-(IBAction)  booleanSections: (id) sender;	

-(IBAction)  kinematicRotationSurface: (id) sender;
-(IBAction)  kinematicRotationBody: (id) sender;
-(IBAction)  kinematicExtrudeSurface: (id) sender;	
-(IBAction)  kinematicExtrudeBody: (id) sender;
-(IBAction)  kinematicSpiralSurface: (id) sender;
-(IBAction)  kinematicSpiralBody: (id) sender;	
-(IBAction)  kinematicPipeSurface: (id) sender;
-(IBAction)  kinematicPipeBody: (id) sender;

-(IBAction)  surfacesMesh: (id) sender;
-(IBAction)  surfacesFace: (id) sender;
-(IBAction)  surfacesCoons3: (id) sender;	
-(IBAction)  surfacesCoons4: (id) sender;	
-(IBAction)  surfacesLinear: (id) sender;	
-(IBAction)  surfacesFromClips: (id) sender;
-(IBAction)  surfacesBodyFromClips: (id) sender;

-(IBAction)  scenesRoom: (id) sender;
-(IBAction)  scenesDrills: (id) sender;
-(IBAction)  scenesClock: (id) sender;

-(IBAction)  filesSave: (id) sender;
-(IBAction)  filesLoad: (id) sender;
-(IBAction)  filesImportDXF: (id) sender;	
-(IBAction)  filesExportDXF: (id) sender;	
-(IBAction)  filesImportSTL: (id) sender;	
-(IBAction)  filesExportSTL: (id) sender;	
-(IBAction)  objFromTriangles: (id) sender;	
-(IBAction)  TDText1: (id) sender;	
-(IBAction)  TDText2: (id) sender;	
-(IBAction)  TDDim1: (id) sender;	
-(IBAction)  TDDim2: (id) sender;	

-(IBAction)  surfacesLinear: (id) sender;	
-(IBAction)  surfacesFromClips: (id) sender;
-(IBAction)  surfacesBodyFromClips: (id) sender;


- (void)  FitToScene;
- (void)  RotateCamera:(SIZE)sdvig;
- (void)  TranslateCamera:(SG_VECTOR)sdvig;
- (void)  ZoomCamera:(SIZE)sdvig;

- (void) mouseDown:(NSEvent *)theEvent;

- (void) mouseUp:(NSEvent *)theEvent;
- (void) mouseDragged:(NSEvent *)theEvent;

- (void) drawRect:(NSRect)rect;

- (void) prepareOpenGL;

- (BOOL) acceptsFirstResponder;
- (BOOL) becomeFirstResponder;
- (BOOL) resignFirstResponder;

- (id) initWithFrame: (NSRect) frameRect;

@end
