
#import <Cocoa/Cocoa.h>

#import "BasicOpenGLView.h"

#import "Painter.h"
#import "Scenes/Scenes.h"

// ==================================


static GLfloat g_fMouseRotateMult = 0.5f; // Mouse 'Rotation' multiplier 
static GLfloat g_fMouseScaleMult  = 0.01f;  // Mouse 'Scale' multiplier

// ---------------------------------

// draw our simple cube based on current modelview and projection matrices
static void drawScene ()
{
	sgCObject*  curObj = sgGetScene()->GetObjectsList()->GetHead();
	while (curObj)
	{
		if (true)
		{
			Painter::DrawObject(GL_RENDER,curObj,false);
			
			if ((curObj->GetAttribute(SG_OA_DRAW_STATE) & SG_DS_GABARITE))
			{
				SG_POINT a1,a2;
				curObj->GetGabarits(a1,a2);
				Painter::DrawGabariteBox(a1,a2);
			}
		}
		curObj = sgGetScene()->GetObjectsList()->GetNext(curObj);
	}
	
	
}

// ===================================

@implementation BasicOpenGLView

// pixel format definition
+ (NSOpenGLPixelFormat*) basicPixelFormat
{
    NSOpenGLPixelFormatAttribute attributes [] = {
        NSOpenGLPFAWindow,
        NSOpenGLPFADoubleBuffer,	// double buffered
        NSOpenGLPFADepthSize, (NSOpenGLPixelFormatAttribute)16, // 16 bit depth buffer
        (NSOpenGLPixelFormatAttribute)nil
    };
    return [[[NSOpenGLPixelFormat alloc] initWithAttributes:attributes] autorelease];
}

// ---------------------------------

// update the projection matrix based on camera and view info
- (void) updateProjection
{
	GLdouble ratio, radians, wd2;
	GLdouble left, right, top, bottom, near, far;

    [[self openGLContext] makeCurrentContext];

	// Set the matrix mode to MODELVIEW and load
	// the identity matrix
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	// Set our camera position and save its transformation matrix
    m_camera.PositionCamera();
}

// ---------------------------------

// updates the contexts model view matrix for object and camera moves
- (void) updateModelView
{
    [[self openGLContext] makeCurrentContext];
	
	// Clear the buffers
	//glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	// Set the matrix mode to MODELVIEW and load
	// the identity matrix
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	// Set our camera position and save its transformation matrix
    m_camera.PositionCamera();
}

// ---------------------------------

// handles resizing of GL need context update and if the window dimensions change, a
// a window dimension update, reseting of viewport and an update of the projection matrix
- (void) resizeGL
{
	NSRect rectView = [self bounds];
	
	// ensure camera knows size changed
	if ((m_camera.m_iScreenHeight != rectView.size.height) ||
	    (m_camera.m_iScreenWidth != rectView.size.width)) {
		m_camera.ResetView(rectView.size.width, rectView.size.height);
		[self FitToScene];
		[self updateProjection];  // update projection matrix
	}
}

// ---------------------------------

// sets the camera data to initial conditions
- (void) resetCamera
{
	m_camera.ReInit();
}

#pragma mark ---- Text Drawing ----

// these functions create or update GLStrings one should expect to have to regenerate the image, bitmap and texture when the string changes thus these functions are not particularly light weight

// ---------------------------------


// ---------------------------------



-(IBAction)  primitivesPoints: (id) sender
{
   OnBnClickedPrPoints();
	[self FitToScene];
	[self setNeedsDisplay: YES];
	
}

-(IBAction)  primitivesLines: (id) sender
{
	OnBnClickedPrLines();
	[self FitToScene];
	[self setNeedsDisplay: YES];
}


-(IBAction)  primitivesCircles: (id) sender
{
	OnBnClickedPrCircles();
	[self FitToScene];
	[self setNeedsDisplay: YES];
}


-(IBAction)  primitivesArcs: (id) sender
{
	OnBnClickedPrArcs();
	[self FitToScene];
	[self setNeedsDisplay: YES];
}


-(IBAction)  primitivesSplines: (id) sender
{
	OnBnClickedPrSplines();
	[self FitToScene];
	[self setNeedsDisplay: YES];
}


-(IBAction)  primitivesContours: (id) sender
{
	OnBnClickedPrContours();
	[self FitToScene];
	[self setNeedsDisplay: YES];
}


-(IBAction)  primitivesEquidistans: (id) sender
{
	OnBnClickedPrEquidi();
[self FitToScene];
	[self setNeedsDisplay: YES];
}


-(IBAction)  primitivesBoxes: (id) sender
{
	OnBnClickedPrBoxes();
	[self FitToScene];
	[self setNeedsDisplay: YES];
}


-(IBAction)  primitivesSpheres: (id) sender
{
	OnBnClickedPrSpheres();
	[self FitToScene];
	[self setNeedsDisplay: YES];
}


-(IBAction)  primitivesCylinders: (id) sender
{
	OnBnClickedPrCyls();
	[self FitToScene];
	[self setNeedsDisplay: YES];
}


-(IBAction)  primitivesCones: (id) sender
{
	OnBnClickedPrCones();
	[self FitToScene];
	[self setNeedsDisplay: YES];
}


-(IBAction)  primitivesEllipsoids: (id) sender
{
	OnBnClickedPrEllipsoids();
	[self FitToScene];
	[self setNeedsDisplay: YES];
}


-(IBAction)  primitivesTors: (id) sender
{
	OnBnClickedPrTors();
	[self FitToScene];
	[self setNeedsDisplay: YES];
}


-(IBAction)  primitivesSphericBands: (id) sender
{
	OnBnClickedPrSphericBands();
	[self FitToScene];
	[self setNeedsDisplay: YES];
}



-(IBAction)  booleanIntersection: (id) sender
{
	OnBnClickedBInters();
	[self FitToScene];
	[self setNeedsDisplay: YES];
}


-(IBAction)  booleanUnion: (id) sender
{
	OnBnClickedBUnion();
	[self FitToScene];
	[self setNeedsDisplay: YES];
}


-(IBAction)  booleanSub: (id) sender
{
	OnBnClickedBSub();
	[self FitToScene];
	[self setNeedsDisplay: YES];
}


-(IBAction)  booleanIntersectionContours: (id) sender
{
	OnBnClickedBIntCont();
	[self FitToScene];
	[self setNeedsDisplay: YES];
}


-(IBAction)  booleanSections: (id) sender
{
	OnBnClickedBSection();
	[self FitToScene];
	[self setNeedsDisplay: YES];
}



-(IBAction)  kinematicRotationSurface: (id) sender
{
	OnBnClickedKinRotSurf();
	[self FitToScene];
	[self setNeedsDisplay: YES];
}


-(IBAction)  kinematicRotationBody: (id) sender
{
	 OnBnClickedKinRotBod();
	[self FitToScene];
	[self setNeedsDisplay: YES];
}


-(IBAction)  kinematicExtrudeSurface: (id) sender
{
	OnBnClickedKinExtrSurf();
	[self FitToScene];
	[self setNeedsDisplay: YES];
}


-(IBAction)  kinematicExtrudeBody: (id) sender
{
	OnBnClickedKinExtrBod();
	[self FitToScene];
	[self setNeedsDisplay: YES];
}


-(IBAction)  kinematicSpiralSurface: (id) sender
{
	OnBnClickedKinSpirSurf();
	[self FitToScene];
	[self setNeedsDisplay: YES];
}


-(IBAction)  kinematicSpiralBody: (id) sender
{
	OnBnClickedKinSpirBod();
	[self FitToScene];
	[self setNeedsDisplay: YES];
}


-(IBAction)  kinematicPipeSurface: (id) sender
{
	OnBnClickedKinPipeSurf();
	[self FitToScene];
	[self setNeedsDisplay: YES];
}


-(IBAction)  kinematicPipeBody: (id) sender
{
	OnBnClickedKinPipeBod();
	[self FitToScene];
	[self setNeedsDisplay: YES];
}



-(IBAction)  surfacesMesh: (id) sender
{
	OnBnClickedSurfMesh();
	[self FitToScene];
	[self setNeedsDisplay: YES];
}


-(IBAction)  surfacesFace: (id) sender
{
	OnBnClickedSurfFace();
	[self FitToScene];
	[self setNeedsDisplay: YES];
}


-(IBAction)  surfacesCoons3: (id) sender
{
	OnBnClickedSurfCoons3();
	[self FitToScene];
	[self setNeedsDisplay: YES];
}


-(IBAction)  surfacesCoons4: (id) sender
{
	OnBnClickedSurfCoons4();
	[self FitToScene];
	[self setNeedsDisplay: YES];
}


-(IBAction)  surfacesLinear: (id) sender
{
	OnBnClickedSurfLinear();
	[self FitToScene];
	[self setNeedsDisplay: YES];
}


-(IBAction)  surfacesFromClips: (id) sender
{
	OnBnClickedSurfFromSect();
	[self FitToScene];
	[self setNeedsDisplay: YES];
}


-(IBAction)  surfacesBodyFromClips: (id) sender
{
	OnBnClickedSurfBodyFromSect();
	[self FitToScene];
	[self setNeedsDisplay: YES];
}



-(IBAction)  scenesRoom: (id) sender
{
	OnBnClickedScenesRoom();
	[self FitToScene];
	[self setNeedsDisplay: YES];
}


-(IBAction)  scenesDrills: (id) sender
{
	OnBnClickedScenesDrills();
	[self FitToScene];
	[self setNeedsDisplay: YES];
}


-(IBAction)  scenesClock: (id) sender
{
	OnBnClickedScenesClock();
	[self FitToScene];
	[self setNeedsDisplay: YES];
}

-(IBAction)  filesSave: (id) sender
{
	OnBnClickedSave();
}

-(IBAction)  filesLoad: (id) sender
{
	OnBnClickedLoad();
	[self FitToScene];
	[self setNeedsDisplay: YES];

}

-(IBAction)  filesImportDXF: (id) sender
{
	OnBnClickedImportDXF();
	[self FitToScene];
	[self setNeedsDisplay: YES];
}

-(IBAction)  filesExportDXF: (id) sender
{
		OnBnClickedExportDXF();
}

-(IBAction)  filesImportSTL: (id) sender
{
     OnBnClickedImportSTL();
	[self FitToScene];
	[self setNeedsDisplay: YES];
	
}

-(IBAction)  filesExportSTL: (id) sender
{
		OnBnClickedExportSTL();
}

-(IBAction)  objFromTriangles: (id) sender
{
	OnBnClickedFromTriangles();
	[self FitToScene];
	[self setNeedsDisplay: YES];
}

-(IBAction)  TDText1: (id) sender
{
	OnBnClickedText1();
	[self FitToScene];
	[self setNeedsDisplay: YES];
}

-(IBAction)  TDText2: (id) sender
{
	OnBnClickedText2();
	[self FitToScene];
	[self setNeedsDisplay: YES];
}

-(IBAction)  TDDim1: (id) sender
{
	OnBnClickedDim1();
	[self FitToScene];
	[self setNeedsDisplay: YES];
}

-(IBAction)  TDDim2: (id) sender
{
	OnBnClickedDim2();
	[self FitToScene];
	[self setNeedsDisplay: YES];
}



- (void)  FitToScene
{
	SG_POINT a1,a2;
	sgGetScene()->GetGabarits(a1,a2);
	m_camera.FitBounds(a1.x,a1.y,a1.z,a2.x,a2.y,a2.z);
}

- (void)  RotateCamera:(SIZE)sdvig
{
	double rx, ry, rz;
	double x, y, z;
	
	rx = ry = rz = 0;
	
	m_camera.GetRotationAboutLookAt(&x, &y, &z);
	
	rz = z - sdvig.cx*g_fMouseRotateMult;
	rx = x + sdvig.cy*g_fMouseRotateMult;
	ry = y;
	
	m_camera.SetRotationAboutLookAt(rx, ry, rz);
}

- (void)  TranslateCamera:(SG_VECTOR)sdvig
{
	GLdouble px, py, pz;
	GLdouble x,  y,  z;
	
	// Calculate the camera's new look 'At' position
	m_camera.GetLookAtPos(&px, &py, &pz);
	x = px - sdvig.x;
	y = py - sdvig.y;
	z = pz - sdvig.z;
	m_camera.SetLookAtPos(x, y, z);
	
	// Calculate the camera's new 'Eye' position
	m_camera.GetEyePos(&px, &py, &pz);
	x = px - sdvig.x;
	y = py - sdvig.y;
	z = pz - sdvig.z;
	m_camera.SetEyePos(x, y, z);
	
	
	// the identity matrix
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	
	// Build the new ModelView matrix using the new camera coordinates
	m_camera.PositionCamera();
}

- (void)  ZoomCamera:(SIZE)sdvig
{
	// Get the camera's Focal length
	if (m_camera.m_bPerspective)
	{
		GLdouble fLength;
		
		fLength = m_camera.GetFocalLength();
		fLength += sdvig.cy*g_fMouseScaleMult*(m_camera.m_fFocalLength/2);
		if(fLength < 1.0f)
			fLength = 1.0f;
		if(fLength > m_camera.m_fFar)
			fLength = m_camera.m_fFar;
		
		// Set the camera's new Focal length
		m_camera.SetFocalLength(fLength, TRUE);
	}
	else
	{
		m_camera.m_fFovY+=sdvig.cy;
		m_camera.ResetView();
	}
}


#pragma mark ---- Method Overrides ----

// ---------------------------------

- (void)mouseDown:(NSEvent *)theEvent // trackball
{
	NSPoint location = [self convertPoint:[theEvent locationInWindow] fromView:nil];
		location.y = m_camera.m_iScreenHeight - location.y;
	m_capture_x = location.x;
	m_capture_y = location.y;

}

// ---------------------------------

- (void)mouseUp:(NSEvent *)theEvent
{
	
	m_capture_x = -1;
	m_capture_y = -1;
}
// ---------------------------------

- (void)mouseDragged:(NSEvent *)theEvent
{
	NSPoint location = [self convertPoint:[theEvent locationInWindow] fromView:nil];
	location.y = m_camera.m_iScreenHeight - location.y;
		
	if (m_capture_x<0 || m_capture_y<0)
		return ;
	SIZE deltaPos;
	deltaPos.cx = m_capture_x - location.x;
	deltaPos.cy = m_capture_y - location.y;
	[self RotateCamera:deltaPos];
	
	m_capture_x = location.x;
	m_capture_y = location.y;
	
	[self setNeedsDisplay: YES];
}


// ---------------------------------

- (void) drawRect:(NSRect)rect
{		
	// setup viewport and prespective
	[self resizeGL]; // forces projection matrix update (does test for size changes)
	[self updateModelView];  // update model view matrix for object

	// clear our drawable
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	// model view and projection matricies already set

	drawScene(); // draw scene
   		
	[[self openGLContext] flushBuffer];

}

// ---------------------------------

// set initial OpenGL state (current context is set)
// called after context is created
- (void) prepareOpenGL
{
    int swapInt = 1;

    [[self openGLContext] setValues:&swapInt forParameter:NSOpenGLCPSwapInterval]; // set to vbl sync

	// Lights properties
	//float ambientProperties[]  = {0.1f, 0.1f, 0.1f, 1.0f};
	float diffuseProperties[]  = {1.0f, 1.0f, 1.0f, 1.0f};
	float specularProperties[] = {1.0f, 1.0f, 1.0f, 1.0f};
	
	//glLightfv( GL_LIGHT0, GL_AMBIENT, ambientProperties);
	glLightfv( GL_LIGHT0, GL_DIFFUSE,diffuseProperties);
	glLightfv( GL_LIGHT0, GL_SPECULAR, specularProperties);
	glLightModelf(GL_LIGHT_MODEL_TWO_SIDE, 1.0);
	
	glClearColor(1.0,1.0,1.0,1.0f);
	
	glHint(GL_LINE_SMOOTH_HINT,GL_FASTEST);
	
	// Texture
	glEnable(GL_TEXTURE_2D);
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
	
	// Default : lighting
	
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	
	
	// Default : blending
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	
	glEnable(GL_DEPTH_TEST);
	
	//  glEnable(GL_CULL_FACE);	
	
	[self resetCamera];
	
	m_capture_x  = -1,
	m_capture_y =  -1;
	
	[self resizeGL];
	
}

// ---------------------------------

-(id) initWithFrame: (NSRect) frameRect
{
    
    sgInitKernel();
    sgC3DObject::AutoTriangulate(true, SG_DELAUNAY_TRIANGULATION, SG_SEPARATE_BUFFERS);
    
	NSOpenGLPixelFormat * pf = [BasicOpenGLView basicPixelFormat];

	self = [super initWithFrame: frameRect pixelFormat: pf];
    return self;
}

// ---------------------------------

- (BOOL)acceptsFirstResponder
{
  return YES;
}

// ---------------------------------

- (BOOL)becomeFirstResponder
{
  return  YES;
}

// ---------------------------------

- (BOOL)resignFirstResponder
{
  return YES;
}



@end
