//
//  ViewController.swift
//  SwiftDemo
//
//  Copyright (c) 2014 Geometros. All rights reserved.
//

import Cocoa
import SceneKit
import SpriteKit

class ViewController: NSViewController {

    @IBOutlet var scene_view : SCNView!
    
    required init!(coder: NSCoder!)
    {
        super.init(coder: coder)
        
    }
    
    

    
    var _spotLightNode : SCNNode = SCNNode()
    var _cameraNode : SCNNode = SCNNode()
    
    var _sg_objects = NSMutableArray()
    
    var camera:SCNCamera = SCNCamera()
    
    var sceneIndex:CInt = 0
    
    let pi = 3.14159265


    override func loadView() {
        super.loadView()

        sgCore.sg_init_library()
        
        //set the background to back
        scene_view?.backgroundColor = SKColor.white
        
        //setup the scene
        //present it
        
        scene_view?.scene = setupScene()
        scene_view?.antialiasingMode = SCNAntialiasingMode.multisampling4X
        
        scene_view?.pointOfView = _cameraNode
        
        sgCore.test(0)
        fillScene()

        
    }

    override var representedObject: Any? {
        didSet {
        // Update the view, if already loaded.
        }
                                    
    }
    
    func setupScene() -> SCNScene
    {
        // create a new scene
        var  scene:SCNScene = SCNScene()
        
        //global environment
        setupEnvironment(scene)
        
        //create a main camera
        camera.zNear = 200
        camera.zFar = 2000
        
        _cameraNode.camera = camera
        
        _cameraNode.position = SCNVector3Make(0, 500, 500)
        _cameraNode.rotation  = SCNVector4Make(1, 0, 0, 5.5890)
        scene.rootNode.addChildNode(_cameraNode)

        return scene
    }
    
    func setupEnvironment(_ scene:SCNScene)
    {
        // add an ambient light
        var ambientLight:SCNNode = SCNNode()
        ambientLight.light = SCNLight()
        ambientLight.light?.type = SCNLight.LightType.ambient;
        ambientLight.light?.color = NSColor(white: 0.5, alpha: 0.1)
        scene.rootNode.addChildNode(ambientLight)
        
        //add a key light to the scene
        var lightNode:SCNNode = SCNNode()
        lightNode.light = SCNLight()
        lightNode.light?.type = SCNLight.LightType.spot
        lightNode.light?.castsShadow = false
        lightNode.light?.color = NSColor(white:0.8, alpha:1.0)
        lightNode.position = SCNVector3Make(0, 1000, 1200)
        lightNode.rotation = SCNVector4Make(1, 0, 0, -1.122)
        lightNode.light?.spotInnerAngle = 0
        lightNode.light?.spotOuterAngle = 100
        lightNode.light?.shadowColor = SKColor.black
        lightNode.light?.zFar = 200
        lightNode.light?.zNear = 100
        scene.rootNode.addChildNode(lightNode)
        
        //keep an ivar for later manipulation
        _spotLightNode = lightNode
        
        //floor
        let floorGeo = SCNFloor()
        floorGeo.firstMaterial?.diffuse.contents = "floor.png"
        floorGeo.firstMaterial?.diffuse.contentsTransform = SCNMatrix4MakeScale(1.5, 1.5, 1)//scale the wood texture
        floorGeo.firstMaterial?.locksAmbientWithDiffuse = true;
        
        let floor = SCNNode(geometry: floorGeo)


        
        scene.rootNode.addChildNode(floor)
    }


    @IBAction func PressNext(_ sender : AnyObject)
    {
        clearScene()
        sceneIndex += 1
        if sceneIndex>12
        {
            sceneIndex=0
        }
        sgCore.test(sceneIndex)
        fillScene()
    }
    @IBAction func PressPrevious(_ sender : AnyObject)
    {
        clearScene()
        sceneIndex -= 1
        if sceneIndex<0
        {
            sceneIndex=12
        }
        sgCore.test(sceneIndex)
        fillScene()
        
    }
    @IBAction func PressScene(_ sender : AnyObject)
    {
        var tg:CInt = CInt(sender.tag)
        
        sceneIndex = tg
        clearScene()
        sgCore.test(sceneIndex)
        fillScene()
    }
    func clearScene()
    {
        
        var curInd = 0
        while curInd<_sg_objects.count
        {
            var tmpN:SCNNode = _sg_objects[curInd] as! SCNNode
            tmpN.removeFromParentNode()
            curInd += 1
        }
        _sg_objects.removeAllObjects()
        sgCore.sg_scene_clear()
    }
    
    func addOneObject(_ obj:sg_object)
    {
        var ot:CInt = sgCore.sg_object_type(obj)
        
        if ot==OT_3D
        {
            var block:SCNNode = SCNNode()
            
            //block.rotation = SCNVector4Make(1, 0, 0, -pi/2)
            block.transform.m11 = CGFloat(sgCore.sg_get_transform_item(0, of_object: obj));
            block.transform.m12 = CGFloat(sgCore.sg_get_transform_item(1, of_object: obj));
            block.transform.m13 = CGFloat(sgCore.sg_get_transform_item(2, of_object: obj));
            block.transform.m14 = CGFloat(sgCore.sg_get_transform_item(3, of_object: obj));

            block.transform.m21 = CGFloat(sgCore.sg_get_transform_item(4, of_object: obj));
            block.transform.m22 = CGFloat(sgCore.sg_get_transform_item(5, of_object: obj));
            block.transform.m23 = CGFloat(sgCore.sg_get_transform_item(6, of_object: obj));
            block.transform.m24 = CGFloat(sgCore.sg_get_transform_item(7, of_object: obj));

            block.transform.m31 = CGFloat(sgCore.sg_get_transform_item(8, of_object: obj));
            block.transform.m32 = CGFloat(sgCore.sg_get_transform_item(9, of_object: obj));
            block.transform.m33 = CGFloat(sgCore.sg_get_transform_item(10, of_object: obj));
            block.transform.m34 = CGFloat(sgCore.sg_get_transform_item(11, of_object: obj));

            block.transform.m41 = CGFloat(sgCore.sg_get_transform_item(12, of_object: obj));
            block.transform.m42 = CGFloat(sgCore.sg_get_transform_item(13, of_object: obj));
            block.transform.m43 = CGFloat(sgCore.sg_get_transform_item(14, of_object: obj));
            block.transform.m44 = CGFloat(sgCore.sg_get_transform_item(15, of_object: obj));
            
            block.transform = SCNMatrix4Mult(block.transform, SCNMatrix4MakeRotation(-3.14159265/2, 1, 0, 0))


            
            var nTr:CInt = sgCore.sg_get_triangles_count(obj)
            nTr *= 3
            
            //var vert_result = sgCore.sg_get_triangles_vertexes(obj)
            var vert_pointer = UnsafePointer<SCNVector3>(sgCore.sg_get_triangles_vertexes(obj))
            
            //var norm_result = sgCore.sg_get_triangles_normals(obj)
            var norm_pointer = UnsafePointer<SCNVector3>(sgCore.sg_get_triangles_normals(obj))
            
           // var tex_result = sgCore.sg_get_triangles_textures(obj)
            var tex_pointer = UnsafePointer<CGPoint>(sgCore.sg_get_triangles_textures(obj))
            
            var geoArr = [SCNGeometrySource(vertices: vert_pointer, count: Int(nTr)),
                SCNGeometrySource(normals: norm_pointer, count: Int(nTr)),
                SCNGeometrySource(textureCoordinates: tex_pointer, count: Int(nTr))]
            
            var arrayElls = [Int32](repeating: 0, count: Int(nTr))
            for i in 0...Int32(nTr-1) { arrayElls[Int(i)] = i }
            
            var ells:SCNGeometryElement =  SCNGeometryElement(data:Data(bytes: UnsafePointer<UInt8>(arrayElls), count: sizeof(Int32) * Int(nTr)),
                primitiveType:SCNGeometryPrimitiveType.triangles,
                primitiveCount:Int(nTr/3),
                bytesPerIndex:sizeof(Int32));
            
            var geoEll = [ells];
            
            _sg_objects.add(block)
            
            //attach a box of 5x5x5
            block.geometry = SCNGeometry(sources:geoArr, elements:geoEll);
            
            let objColor = sgCore.sg_object_color(obj)
            var textureName = "white.jpg"
            
            if objColor==COLOR_GRAY || objColor==COLOR_GRAY_TR
            {
                textureName = "gray.jpg"
            }
            else if objColor==COLOR_GREEN || objColor==COLOR_GREEN_TR
            {
                textureName = "green.jpg"
            }
            else if objColor==COLOR_BLUE || objColor==COLOR_BLUE_TR
            {
                textureName = "blue.jpg"
            }
            else if objColor==COLOR_RED || objColor==COLOR_RED_TR
            {
                textureName = "red.jpg"
                block.geometry?.firstMaterial?.specular.contents = NSColor.white;
            }
            else if objColor==COLOR_YELLOW || objColor==COLOR_YELLOW_TR
            {
                textureName = "yellow.jpg"
            }
            
            block.geometry?.firstMaterial?.diffuse.contents = textureName  //imageName
            block.geometry?.firstMaterial?.diffuse.contentsTransform = SCNMatrix4MakeScale(0.05, 0.05, 1) //scale the wood texture
            
            if objColor>=COLOR_GRAY_TR
            {
               block.geometry?.firstMaterial?.transparency = 0.3
            }
            else
            {
                block.geometry?.firstMaterial?.isDoubleSided = true
            }
            
            // block.geometry.firstMaterial.normal.contents = @"nm.png";
            // block.geometry.firstMaterial.normal.mipFilter = SCNFilterModeLinear;
            // block.geometry.firstMaterial.normal.contentsTransform = SCNMatrix4MakeScale(0.05, 0.05, 1); //scale the wood texture
            
            scene_view.scene?.rootNode.addChildNode(block);
        }
        else
        {
            if ot==OT_2D
            {
                var block:SCNNode = SCNNode()
                
                block.position = SCNVector3Make(0, 1, 0)
                block.rotation = SCNVector4Make(1, 0, 0, -3.14159265/2)
                
                
                var nPnts:CInt = sgCore.sg_get_points_count(obj)
                
               // var vert_result = sgCore.sg_get_points(obj)
                var vert_pointer = UnsafePointer<SCNVector3>(sgCore.sg_get_points(obj))
                
                var geoArr = [SCNGeometrySource(vertices: vert_pointer, count: Int(nPnts))]
                
                var cnt = 0
                if sgCore.sg_is_strip(obj)
                {
                    cnt = 2*Int(nPnts)
                }
                else
                {
                    cnt = Int(nPnts)
                }
                
                var subParam = 0
                if sgCore.sg_is_close(obj)
                {
                    subParam = -1
                }
                
                var stripNonCloseParam = 0
                if !sgCore.sg_is_close(obj) && sgCore.sg_is_strip(obj)
                {
                    stripNonCloseParam = -2
                }
                
                
                var arrayElls = [Int32](repeating: 0, count: cnt+stripNonCloseParam)
                for i in 1...(cnt+subParam+stripNonCloseParam-1)
                {
                    if sgCore.sg_is_strip(obj)
                    {
                        var reminder = i%2
                        if reminder==0
                        {
                            arrayElls[Int(i)] = Int32(i/2)
                        }
                        else
                        {
                            arrayElls[Int(i)] = Int32(i/2+1)
                        }
                    }
                    else
                    {
                        arrayElls[Int(i)] = Int32(i)
                    }
                }
                
                var ells:SCNGeometryElement =  SCNGeometryElement(data:Data(bytes: UnsafePointer<UInt8>(arrayElls), count: (cnt+stripNonCloseParam) * sizeof(Int32) ),
                    primitiveType:SCNGeometryPrimitiveType.line,
                    primitiveCount:cnt-1+stripNonCloseParam,
                    bytesPerIndex:sizeof(Int32));
                
                var geoEll = [ells];
                
                
                _sg_objects.add(block)
                
                
                //attach a box of 5x5x5
                block.geometry = SCNGeometry(sources:geoArr, elements:geoEll);
                // block.geometry.firstMaterial.diffuse.contents = "red.jpg" //imageName;
                // block.geometry.firstMaterial.diffuse.contentsTransform = SCNMatrix4MakeScale(0.05, 0.05, 1); //scale the wood texture
                // block.geometry.firstMaterial.diffuse.mipFilter = SCNFilterMode.Linear;
                
                scene_view.scene?.rootNode.addChildNode(block);
            }
            else
            {
                if ot==OT_GROUP
                {
                    var curObj:sg_object = sgCore.sg_get_group_first_object(obj)
                    var curInd = 0
                    
                    while curInd<Int(sgCore.sg_get_group_sub_objects_count(obj))
                    {
                        addOneObject(curObj)
                        curObj = sgCore.sg_get_group_next_object(obj, after: curObj)
                        curInd += 1
                    }
                }
            }
        }
    }
    func fillScene()
    {
        var curObj:sg_object = sgCore.sg_scene_get_first_object()
        var curInd = 0
        
        while curInd<Int(sgCore.sg_scene_get_objects_count())
        {
            addOneObject(curObj)
            curObj = sgCore.sg_scene_get_next_object(curObj)
            curInd += 1
        }
    }


}

