#import <QuartzCore/QuartzCore.h>
#import <OpenGLES/EAGLDrawable.h>

#import "EAGLView.h"


#import "Painter.h"
#import "Scenes/Scenes.h"

// ==================================
// ---------------------------------

// draw our simple cube based on current modelview and projection matrices
static void drawScene ()
{
	sgCObject*  curObj = sgGetScene()->GetObjectsList()->GetHead();
	while (curObj)
	{
		if (true)
		{
			Painter::DrawObject(curObj,false);
			
		}
		curObj = sgGetScene()->GetObjectsList()->GetNext(curObj);
	}
	
	
}


@interface EAGLView ()

@property (nonatomic, retain) EAGLContext *context;
@property (nonatomic, assign) NSTimer *animationTimer;

- (BOOL)createFramebuffer;
- (void)destroyFramebuffer;
- (void)initGL;
- (void)initTextures;
- (void)gameLoop;
- (void)updateScene:(float)delta;
- (void)renderScene;
- (void)render_main;

@end


@implementation EAGLView

@synthesize context;
@synthesize animationTimer;
@synthesize animationInterval;

+ (Class)layerClass {
	return [CAEAGLLayer class];
}

- (id)initWithCoder:(NSCoder*)coder {

	if((self = [super initWithCoder:coder])) {

		CAEAGLLayer *eaglLayer = (CAEAGLLayer*)self.layer;

		eaglLayer.opaque = YES;
		eaglLayer.drawableProperties = [NSDictionary dictionaryWithObjectsAndKeys:
										[NSNumber numberWithBool:NO], kEAGLDrawablePropertyRetainedBacking,
										kEAGLColorFormatRGBA8, kEAGLDrawablePropertyColorFormat, nil];

		context = [[EAGLContext alloc] initWithAPI:kEAGLRenderingAPIOpenGLES1];

		if(!context || ![EAGLContext setCurrentContext:context]) {
			[self release];
			return nil;
		}

		animationInterval = 1.0 / kFPS;
		
		currentTime = CFAbsoluteTimeGetCurrent();
		lastTime = currentTime;
		
		xRotation = 0.0f;
		yRotation = 0.0f;
		
		[self setMultipleTouchEnabled:YES];
		
		[self initGL];
		[self initTextures];
	}
	return self;
}

void gluPerspective(GLfloat fovy, GLfloat aspect, GLfloat zNear, GLfloat zFar) {
	
	GLfloat xmin, xmax, ymin, ymax;
	
	ymax = zNear * tan(fovy * M_PI / 360.0f);
	ymin = -ymax;
	xmin = ymin * aspect;
	xmax = ymax * aspect;
	
	glFrustumf(xmin, xmax, ymin, ymax, zNear, zFar);
}

- (void)initGL {
	
    CGRect   rect = [self frame];
	
	glEnable(GL_TEXTURE_2D);
	glShadeModel(GL_SMOOTH);
	glClearColor(0.8f, 0.8f, 0.8f, 1.0f);
	glClearDepthf(1.0f);
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);
	
	// lighting
	
	GLfloat lightAmbient[]  = { 0.1f, 0.1f, 0.1f, 1.0f };
	GLfloat lightDiffuse[]  = { 1.0f, 1.0f, 1.0f, 1.0f };
	GLfloat lightPosition[] = { 17.0f, 17.0f, 17.0f, 1.0f };
	
	glLightfv(GL_LIGHT1, GL_AMBIENT, lightAmbient);
	glLightfv(GL_LIGHT1, GL_DIFFUSE, lightDiffuse);
	glLightfv(GL_LIGHT1, GL_POSITION, lightPosition);
	glEnable(GL_LIGHT1);
	
	lightingEnabled = YES;
	glEnable(GL_LIGHTING);
	
	// materials
	
	GLfloat materialAmbient[] = { 0.2f, 0.2f, 0.2f, 1.0f };
	GLfloat materialDiffuse[] = { 0.8f, 0.8f, 0.8f, 1.0f };
	
	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, materialAmbient);
	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, materialDiffuse);
    
    OnBnClickedBInters();
}

- (void)initTextures {

	antialiasEnabled = YES;
	[Texture2D setAntiAliasTexParameters];
	
	cubeTexture = [[Texture2D alloc] initWithImage:[UIImage imageNamed:@"cube.jpg"]];
}

- (void) setFrame:(CGRect)frame
{
    // Call the parent class to move the view
    [super setFrame:frame];
    
    glViewport(0, 0, frame.size.width, frame.size.height);
	
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	
	GLfloat aspect = (GLfloat)frame.size.width / (GLfloat)frame.size.height;
	gluPerspective(45.0, aspect, 1.0, 500.0);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
	
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	
}

- (void)gameLoop {

	float delta;
	currentTime = CFAbsoluteTimeGetCurrent();
	delta = (currentTime - lastTime);
	lastTime = currentTime;

	[self updateScene:delta];
	[self renderScene];
}

- (void)updateScene:(float)delta {

	yRotation += 50.0f * delta;
	
	static float amplitude = 0.0f;
	if(amplitude < 80.0f) amplitude += 5.0f * delta;
	xRotation = amplitude * (float)sin(0.4 * currentTime) * 2.0f;
}

- (void)renderScene {
    
	[EAGLContext setCurrentContext:context];
	
	glBindFramebufferOES(GL_FRAMEBUFFER_OES, viewFramebuffer);
	
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();
	
	[self render_main];
	
	glBindRenderbufferOES(GL_RENDERBUFFER_OES, viewRenderbuffer);
	[context presentRenderbuffer:GL_RENDERBUFFER_OES];
}

static void normalize(float v[3])
{
    float r;
    
    r = sqrt( v[0]*v[0] + v[1]*v[1] + v[2]*v[2] );
    if (r == 0.0) return;
    
    v[0] /= r;
    v[1] /= r;
    v[2] /= r;
}

static void cross(float v1[3], float v2[3], float result[3])
{
    result[0] = v1[1]*v2[2] - v1[2]*v2[1];
    result[1] = v1[2]*v2[0] - v1[0]*v2[2];
    result[2] = v1[0]*v2[1] - v1[1]*v2[0];
}
static void __gluMakeIdentityf(GLfloat m[16])
{
    m[0+4*0] = 1; m[0+4*1] = 0; m[0+4*2] = 0; m[0+4*3] = 0;
    m[1+4*0] = 0; m[1+4*1] = 1; m[1+4*2] = 0; m[1+4*3] = 0;
    m[2+4*0] = 0; m[2+4*1] = 0; m[2+4*2] = 1; m[2+4*3] = 0;
    m[3+4*0] = 0; m[3+4*1] = 0; m[3+4*2] = 0; m[3+4*3] = 1;
}
void  gluLookAt(GLfloat eyex, GLfloat eyey, GLfloat eyez, GLfloat centerx,
          GLfloat centery, GLfloat centerz, GLfloat upx, GLfloat upy,
          GLfloat upz)
{
    float forward[3], side[3], up[3];
    GLfloat m[4][4];
    
    forward[0] = centerx - eyex;
    forward[1] = centery - eyey;
    forward[2] = centerz - eyez;
    
    up[0] = upx;
    up[1] = upy;
    up[2] = upz;
    
    normalize(forward);
    
    /* Side = forward x up */
    cross(forward, up, side);
    normalize(side);
    
    /* Recompute up as: up = side x forward */
    cross(side, forward, up);
    
    __gluMakeIdentityf(&m[0][0]);
    m[0][0] = side[0];
    m[1][0] = side[1];
    m[2][0] = side[2];
    
    m[0][1] = up[0];
    m[1][1] = up[1];
    m[2][1] = up[2];
    
    m[0][2] = -forward[0];
    m[1][2] = -forward[1];
    m[2][2] = -forward[2];
    
    glMultMatrixf(&m[0][0]);
    glTranslatef(-eyex, -eyey, -eyez);
}

- (void)render_main {
	    
    SG_POINT  minScene, maxScene;
    sgGetScene()->GetGabarits(minScene, maxScene);

    SG_POINT midPnt;
    midPnt.x = (maxScene.x+minScene.x)/2.0;
    midPnt.y = (maxScene.y+minScene.y)/2.0;
    midPnt.z = (maxScene.z+minScene.z)/2.0;
    
    if (fabs(maxScene.z-minScene.z)<5)
        maxScene.z = 8;

    gluLookAt(maxScene.x*2, maxScene.y*2 , maxScene.z*2, maxScene.x, maxScene.y , maxScene.z, 0, 0, 1);
    
    glRotatef(yRotation, 0.0f, 0.0f, 1.0f);
    glTranslatef(-midPnt.x, -midPnt.y , -midPnt.z);


	
    
//	glBindTexture(GL_TEXTURE_2D, cubeTexture.name);
    
    drawScene();
	
	
}

- (void)layoutSubviews {
	[EAGLContext setCurrentContext:context];
	[self destroyFramebuffer];
	[self createFramebuffer];
	[self renderScene];
}

- (BOOL)createFramebuffer {
    
	glGenFramebuffersOES(1, &viewFramebuffer);
	glGenRenderbuffersOES(1, &viewRenderbuffer);

	glBindFramebufferOES(GL_FRAMEBUFFER_OES, viewFramebuffer);
	glBindRenderbufferOES(GL_RENDERBUFFER_OES, viewRenderbuffer);
	[context renderbufferStorage:GL_RENDERBUFFER_OES fromDrawable:(CAEAGLLayer*)self.layer];
	glFramebufferRenderbufferOES(GL_FRAMEBUFFER_OES, GL_COLOR_ATTACHMENT0_OES, GL_RENDERBUFFER_OES, viewRenderbuffer);

	glGetRenderbufferParameterivOES(GL_RENDERBUFFER_OES, GL_RENDERBUFFER_WIDTH_OES, &backingWidth);
	glGetRenderbufferParameterivOES(GL_RENDERBUFFER_OES, GL_RENDERBUFFER_HEIGHT_OES, &backingHeight);

	glGenRenderbuffersOES(1, &depthRenderbuffer);
	glBindRenderbufferOES(GL_RENDERBUFFER_OES, depthRenderbuffer);
	glRenderbufferStorageOES(GL_RENDERBUFFER_OES, GL_DEPTH_COMPONENT16_OES, backingWidth, backingHeight);
	glFramebufferRenderbufferOES(GL_FRAMEBUFFER_OES, GL_DEPTH_ATTACHMENT_OES, GL_RENDERBUFFER_OES, depthRenderbuffer);
	
	if(glCheckFramebufferStatusOES(GL_FRAMEBUFFER_OES) != GL_FRAMEBUFFER_COMPLETE_OES) {
		NSLog(@"failed to make complete framebuffer object %x", glCheckFramebufferStatusOES(GL_FRAMEBUFFER_OES));
		return NO;
	}

	return YES;
}

- (void)destroyFramebuffer {
	glDeleteFramebuffersOES(1, &viewFramebuffer);
	viewFramebuffer = 0;
	glDeleteRenderbuffersOES(1, &viewRenderbuffer);
	viewRenderbuffer = 0;
}

- (void)startAnimation {
	self.animationTimer = [NSTimer scheduledTimerWithTimeInterval:animationInterval target:self selector:@selector(gameLoop) userInfo:nil repeats:YES];
}

- (void)stopAnimation {
	self.animationTimer = nil;
}

- (void)setAnimationTimer:(NSTimer *)newTimer {
	[animationTimer invalidate];
	animationTimer = newTimer;
}

- (void)setAnimationInterval:(NSTimeInterval)interval {
    
	animationInterval = interval;
	if(animationTimer) {
		[self stopAnimation];
		[self startAnimation];
	}
}

- (void)dealloc {

	[self stopAnimation];

	if([EAGLContext currentContext] == context) {
		[EAGLContext setCurrentContext:nil];
	}

	[cubeTexture release];
	[context release];  
	[super dealloc];
}

@end
